'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _compileWpy = require('./compile-wpy');

var _compileWpy2 = _interopRequireDefault(_compileWpy);

var _loader = require('./loader');

var _loader2 = _interopRequireDefault(_loader);

var _resolve = require('./resolve');

var _resolve2 = _interopRequireDefault(_resolve);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var currentPath = _util2.default.currentDir;

var appPath = void 0,
    npmPath = void 0,
    src = void 0,
    dist = void 0;

exports.default = {
    resolveDeps: function resolveDeps(code, type, opath) {
        var _this = this;

        _util2.default.Log('resolveDeps,opath=', opath.base);
        var params = _cache2.default.getParams();
        var config = _cache2.default.getConfig();
        var wpyExt = params.wpyExt;
        var npmInfo = opath.npm;

        return code.replace(/require\(['"]([\w\d_\-\.\/@]+)['"]\)/ig, function (match, lib) {

            var resolved = lib;

            _util2.default.Log('require lib', lib);

            var target = '',
                source = '',
                ext = '',
                needCopy = false;

            if (config.output === 'ant' && lib === 'wepy') {
                lib = 'wepy-ant';
            }
            lib = _resolve2.default.resolveAlias(lib);
            if (_path2.default.isAbsolute(lib)) {
                source = lib;
                target = _util2.default.getDistPath(source);
            } else if (lib[0] === '.') {
                // require('./something'');
                source = _path2.default.join(opath.dir, lib); // e:/src/util
                if (type === 'npm') {
                    target = _path2.default.join(npmPath, _path2.default.relative(npmInfo.modulePath, source));
                    needCopy = true;
                } else {
                    // e:/dist/util
                    target = _util2.default.getDistPath(source);
                    needCopy = false;
                }
            } else if (lib.indexOf('/') === -1 || // require('asset');
            lib.indexOf('/') === lib.length - 1 || // reqiore('a/b/something/')
            lib[0] === '@' && lib.indexOf('/') !== -1 && lib.lastIndexOf('/') === lib.indexOf('/') // require('@abc/something')
            ) {

                    var mainFile = _resolve2.default.getMainFile(lib);

                    if (!mainFile) {
                        throw Error('找不到模块: ' + lib + '\n被依赖于: ' + _path2.default.join(opath.dir, opath.base) + '。\n请尝试手动执行 npm install ' + lib + ' 进行安装。');
                    }
                    npmInfo = {
                        lib: lib,
                        dir: mainFile.dir,
                        modulePath: mainFile.modulePath,
                        file: mainFile.file
                    };
                    source = _path2.default.join(mainFile.dir, mainFile.file);
                    target = _path2.default.join(npmPath, lib, mainFile.file);

                    lib += _path2.default.sep + mainFile.file;
                    ext = '';
                    needCopy = true;
                } else {
                // require('babel-runtime/regenerator')
                var o = _resolve2.default.walk(lib);
                source = _path2.default.join(o.modulePath, lib);
                target = _path2.default.join(npmPath, lib);
                ext = '';
                needCopy = true;
            }

            if (_util2.default.isFile(source + wpyExt)) {
                ext = '.js';
            } else if (_util2.default.isFile(source + '.js')) {
                ext = '.js';
            } else if (_util2.default.isDir(source) && _util2.default.isFile(source + _path2.default.sep + 'index.js')) {
                ext = _path2.default.sep + 'index.js';
            } else if (_util2.default.isFile(source)) {
                ext = '';
            } else {
                throw '找不到文件: ' + source;
            }
            source += ext;
            target += ext;
            lib += ext;
            resolved = lib;

            // 第三方组件
            if (/\.wpy$/.test(resolved)) {
                target = target.replace(/\.wpy$/, '') + '.js';
                resolved = resolved.replace(/\.wpy$/, '') + '.js';
                lib = resolved;
            }

            if (needCopy) {
                if (!_cache2.default.checkBuildCache(source)) {
                    _cache2.default.setBuildCache(source);
                    _util2.default.log('依赖: ' + _path2.default.relative(process.cwd(), target), '拷贝');
                    var newOpath = _path2.default.parse(source);
                    newOpath.npm = npmInfo;
                    _this.compile('js', null, 'npm', newOpath);
                }
            }
            if (type === 'npm') {
                if (lib[0] !== '.') {
                    resolved = _path2.default.join('..' + _path2.default.sep, _path2.default.relative(opath.dir, npmInfo.modulePath), lib);
                } else {
                    if (lib[0] === '.' && lib[1] === '.') resolved = './' + resolved;
                }
            } else {
                resolved = _path2.default.relative(_util2.default.getDistPath(opath, opath.ext, src, dist), target);
            }
            resolved = resolved.replace(/\\/g, '/').replace(/^\.\.\//, './');

            //处理ocomp组件规则
            var ocomp_path = _util2.default.oCompRealPath(source);
            if (ocomp_path) {
                // console.log("ocomp_path:", ocomp_path)
                resolved = ocomp_path.replace(/\.vue/i, ".js");
                resolved = _path2.default.relative(opath.dir, resolved);
            }

            return 'require(\'' + resolved + '\')';
        });
    },
    npmHack: function npmHack(opath, code) {
        // 一些库（redux等） 可能会依赖 process.env.NODE_ENV 进行逻辑判断
        // 这里在编译这一步直接做替换 否则报错
        code = code.replace(/process\.env\.NODE_ENV/g, JSON.stringify(process.env.NODE_ENV));
        switch (opath.base) {
            case 'lodash.js':
            case '_global.js':
                code = code.replace('Function(\'return this\')()', 'this');
                break;
            case '_html.js':
                code = 'module.exports = false;';
                break;
            case '_microtask.js':
                code = code.replace('if(Observer)', 'if(false && Observer)');
                // IOS 1.10.2 Promise BUG
                code = code.replace('Promise && Promise.resolve', 'false && Promise && Promise.resolve');
                break;
            case '_freeGlobal.js':
                code = code.replace('module.exports = freeGlobal;', 'module.exports = freeGlobal || this;');
        }
        var config = _util2.default.getConfig();
        if (config.output === 'ant' && opath.dir.substr(-19) === 'wepy-async-function') {
            code = '';
        }
        return code;
    },
    compile: function compile(lang, code, type, opath) {
        var _this2 = this;

        var config = _util2.default.getConfig();

        src = _cache2.default.getSrc();
        dist = _cache2.default.getDist();
        npmPath = _path2.default.join(currentPath, dist, 'npm' + _path2.default.sep);

        if (!code) {
            code = _util2.default.readFile(_path2.default.join(opath.dir, opath.base));
            if (code === null) {
                throw '打开文件失败: ' + _path2.default.join(opath.dir, opath.base);
            }
        }

        var compiler = _loader2.default.loadCompiler(lang);

        if (!compiler) {
            return;
        }

        // console.log('编译js:', opath.base);
        //todo: 不要编译组件index.js

        code = code.replace(/暴露[\n\s\r]*?import { register } from 'utils'/g, '');

        if (opath.base.match(/\.json$/)) {
            _util2.default.Log('json', opath);
            code = 'module.exports = ' + code;
        }

        var js = code;
        compiler(code, config.compilers[lang] || {}).then(function (compileResult) {
            // console.log("compileResult for ", opath)
            var sourceMap = void 0;
            if (typeof compileResult === 'string') {
                code = compileResult;
            } else {
                sourceMap = compileResult.map;
                code = compileResult.code;
            }

            _util2.default.Log('opath ,type', opath.base, type, type === 'page');
            if (type !== 'npm') {
                if (type === 'page' || type === 'app') {
                    code = code.replace(/(export\s+default\s+|module\.exports\s+=\s+|exports\.default\s+=\s+){((\n|.)*)}/g, function (m, exports, code) {
                        var s = 'var $export$' + opath.name + ' = {' + code + '}';

                        // util.Log('exports', opath.name, exports, code)
                        // console.log('xxopath1',opath.base);
                        if (type === 'page') {
                            var pagePath = _path2.default.join(_path2.default.relative(appPath.dir, opath.dir), opath.name).replace(/\\/ig, '/');
                            s = s + ('\nPage(require(\'wekit\').default.$createPage($export$' + opath.name + ' , \'' + pagePath + '\'));\n');
                        } else {
                            appPath = opath;
                            var appConfig = JSON.stringify(config.appConfig);
                            s = s + ('\nApp(require(\'wekit\').default.$createApp( $export$' + opath.name + ', ' + appConfig + '));\n');
                        }s;

                        return s;
                    });

                    code = code.replace(/exports\.default\s*=\s*(\w+);/ig, function (m, defaultExport) {
                        // util.Log('opath',code,type, opath.dir, opath.base)
                        // util.Log(m, defaultExport)
                        if (defaultExport === 'undefined') {
                            return '';
                        }
                        if (type === 'page') {
                            var pagePath = _path2.default.join(_path2.default.relative(appPath.dir, opath.dir), opath.name).replace(/\\/ig, '/');
                            return '\nPage(require(\'wekit\').default.$createPage(' + defaultExport + ' , \'' + pagePath + '\'));\n';
                        } else {
                            appPath = opath;
                            var appConfig = JSON.stringify(config.appConfig);
                            return '\nApp(require(\'wekit\').default.$createApp(' + defaultExport + ', ' + appConfig + '));\n';
                        }
                    });
                }
            }

            // util.Log('resolveDeps, code=',code);
            code = _this2.resolveDeps(code, type, opath);

            if (type === 'npm' && opath.ext === '.wpy') {
                // 第三方npm组件，后缀恒为wpy
                _compileWpy2.default.compile(opath);
                return;
            }
            var target = void 0;
            if (type !== 'npm') {
                target = _util2.default.getDistPath(opath, 'js');
            } else {
                code = _this2.npmHack(opath, code);
                target = _path2.default.join(npmPath, _path2.default.relative(opath.npm.modulePath, _path2.default.join(opath.dir, opath.base)));
            }
            if (sourceMap) {
                sourceMap.sources = [opath.name + '.js'];
                sourceMap.file = opath.name + '.js';
                var Base64 = require('js-base64').Base64;
                code += '\r\n//# sourceMappingURL=data:application/json;charset=utf-8;base64,' + Base64.encode(JSON.stringify(sourceMap));
            }

            if (opath.base.match(/\.json$/)) {
                target = _path2.default.join(opath.dir, opath.base + '.js');
                _util2.default.Log('target', target);
            }

            var plg = new _loader2.default.PluginHelper(config.plugins, {
                type: type,
                code: code,
                file: target,
                output: function output(p) {
                    _util2.default.output(p.action, p.file);
                },
                done: function done(result) {
                    _util2.default.output('写入', result.file);
                    _util2.default.writeFile(target, result.code);
                }
            });
            // 缓存文件修改时间戳
            _cache2.default.saveBuildCache();
        }).catch(function (e) {
            _util2.default.error('compile-script.js:' + e + e.stack + '\n 源码:' + code);
            process.exit(0);
        });
    }
};
//# sourceMappingURL=compile-script.js.map