'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _loader = require('./loader');

var _loader2 = _interopRequireDefault(_loader);

var _resolve = require('./resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _scoped = require('./style-compiler/scoped');

var _scoped2 = _interopRequireDefault(_scoped);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LANG_MAP = {
    'less': '.less',
    'sass': '.sass;.scss'
};

exports.default = {
    transforCss: function transforCss(content) {

        content = content.replace(/lines:\s*([0-9]+)/g, function (match, lines) {

            var s = 'display:-webkit-box; overflow: hidden; text-overflow: ellipsis; -webkit-line-clamp: ' + lines + '; -webkit-box-orient: vertical';

            return s;
        });

        return content.replace(/{((.|\n)*?)}/g, function (match, css) {
            match = match.replace(/([0-9]+)px/g, function (m, n) {
                return n + 'rpx';
            });
            css = css.replace(/([0-9]+)px/g, function (m, n) {
                return n + 'rpx';
            });

            return match;
        });
    },
    appendBaseCss: function appendBaseCss(content) {
        content = content + '\n\t    .wx-ct {\n\t        display:flex;\n\t        flex-direction:column;\n\t        box-sizing:border-box;\n\t        position:relative;\n\t        border-style: solid;\n\t        border-width: 0rpx;\n\t    }\n\t    ';
        return content;
    },
    compile: function compile(styles, requires, opath, moduleId) {
        var _this = this;

        // console.log('style compile styles=',styles);
        // console.log('style compile requires=',requires);
        // console.log('style compile opath=',opath?opath.base:opath);
        var config = _util2.default.getConfig();
        var src = _cache2.default.getSrc();
        var dist = _cache2.default.getDist();
        var ext = _cache2.default.getExt();
        var isNPM = false;

        var outputExt = config.output === 'ant' ? 'acss' : 'wxss';

        if (typeof styles === 'string') {
            // .compile('less', opath) 这种形式
            opath = requires;
            requires = [];
            moduleId = '';
            styles = [{
                type: styles,
                scoped: false,
                code: _util2.default.readFile(_path2.default.join(opath.dir, opath.base)) || ''
            }];
        }
        var allPromises = [];

        // styles can be an empty array
        styles.forEach(function (style) {
            var lang = style.type || 'css';
            var content = style.code;

            // content = this.transforCss(content)

            var scoped = style.scoped;
            var filepath = style.src ? style.src : _path2.default.join(opath.dir, opath.base);

            var options = Object.assign({}, config.compilers[lang] || {});

            if (lang === 'sass' || lang === 'scss') {
                var indentedSyntax = false;
                options = Object.assign({}, config.compilers.sass || {});

                if (lang === 'sass') {
                    // sass is using indented syntax
                    indentedSyntax = true;
                }
                if (options.indentedSyntax === undefined) {
                    options.indentedSyntax = indentedSyntax;
                }
                lang = 'sass';
            }

            var compiler = _loader2.default.loadCompiler(lang);

            if (!compiler) {
                throw '\u672A\u53D1\u73B0\u76F8\u5173 ' + lang + ' \u7F16\u8BD1\u5668\u914D\u7F6E\uFF0C\u8BF7\u68C0\u67E5wekit.config.js\u6587\u4EF6\u3002';
            }

            // console.log("编译:", lang, content, filepath)
            // console.log('compiler1=',content,options,filepath);
            content = content.trim();
            var p = compiler(content, options || {}, filepath).then(function (css) {
                // console.log('css=', css)
                // console.log("编译Sucess:", lang, content, filepath,css)
                // 处理 scoped
                // console.log('compiler2=',css);
                if (scoped) {
                    // 存在有 scoped 的 style
                    return (0, _scoped2.default)(moduleId, css).then(function (cssContent) {
                        return cssContent;
                    });
                } else {
                    return css;
                }
            }).catch(function (e) {
                console.log("编译Error:", lang, content, filepath);
                console.log(e);
            });
            allPromises.push(p);
        });

        // 父组件没有写style标签，但是有子组件的。
        if (requires.length > 0 && styles.length === 0) {
            allPromises = [Promise.resolve('')];
        }
        // console.log('allPromises=',opath.base,allPromises);
        Promise.all(allPromises).then(function (rets) {
            var allContent = rets.join('');
            // console.log('allContent=',opath.base,allContent,requires);
            if (requires && requires.length) {
                requires.forEach(function (r) {
                    // console.log('r=',r);
                    var comsrc = null;
                    if (_path2.default.isAbsolute(r)) {
                        if (_path2.default.extname(r) === '' && _util2.default.isFile(r + ext)) {
                            comsrc = r + ext;
                        } else {
                            comsrc = _util2.default.findComponent(r);
                        }
                    } else {
                        var lib = _resolve2.default.resolveAlias(r);
                        if (_path2.default.isAbsolute(lib)) {
                            comsrc = lib;
                            if (_path2.default.extname(comsrc) === '') {
                                comsrc += '.' + outputExt;
                            }
                        } else {
                            var o = _resolve2.default.getMainFile(r);
                            comsrc = _path2.default.join(o.dir, o.file);
                            var newOpath = _path2.default.parse(comsrc);
                            newOpath.npm = {
                                lib: r,
                                dir: o.dir,
                                file: o.file,
                                modulePath: o.modulePath
                            };
                            comsrc = _util2.default.getDistPath(newOpath);
                            comsrc = comsrc.replace(ext, '.' + outputExt).replace('' + _path2.default.sep + dist + _path2.default.sep, '' + _path2.default.sep + src + _path2.default.sep);
                        }
                        isNPM = true;
                    }
                    // console.log('r=',r,comsrc);
                    if (!comsrc) {
                        _util2.default.error('找不到组件：' + r + ('\n\u8BF7\u5C1D\u8BD5\u4F7F\u7528 npm install ' + r + ' \u5B89\u88C5'), '错误');
                    } else {
                        var relative = _path2.default.relative(opath.dir + _path2.default.sep + opath.base, comsrc);
                        var code = _util2.default.readFile(comsrc);

                        // if (isNPM || /<style/.test(code)) {
                        if (/\.wpy$/.test(relative)) {
                            // wpy 第三方组件
                            relative = relative.replace(/\.wpy$/, '.' + outputExt);
                        }
                        relative = relative.replace(ext, '.' + outputExt).replace(/\\/ig, '/').replace('../', './');
                        // relative = relative.replace(/\/[^\/]*?wxss/g, function (match) {
                        //    return '/index.wxss'
                        // });
                        allContent = '@import "' + relative + '";\n' + allContent;
                        // }
                    }
                });
            }
            // console.log('allContent2=',opath.base,allContent);
            var target = _util2.default.getDistPath(opath, outputExt, src, dist);

            // console.log('target=',target);


            allContent = _this.transforCss(allContent);

            allContent = _this.appendBaseCss(allContent);

            var plg = new _loader2.default.PluginHelper(config.plugins, {
                type: 'css',
                code: allContent,
                file: target,
                output: function output(p) {
                    _util2.default.output(p.action, p.file);
                },
                done: function done(rst) {
                    // console.log('done=',rst);
                    _util2.default.output('写入', rst.file);
                    _util2.default.writeFile(target, rst.code);
                }
            });
        }).catch(function (e) {
            _util2.default.error('compile-style.js:' + e + e.stack);
        });
    }
};
//# sourceMappingURL=compile-style.js.map