'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _util = require('./util');

var _util2 = _interopRequireDefault(_util);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
    init: function init(config) {
        this.modules = config.modules || ['node_modules'];
        this.alias = config.alias;

        if (typeof this.modules === 'string') {
            this.modules = [this.modules];
        }

        var cwd = process.cwd();

        this.modulePaths = this.modules.map(function (v) {
            if (_path2.default.isAbsolute(v)) {
                return v;
            } else {
                return _path2.default.join(cwd, v);
            }
        });
        this._cacheModules = {};
        this._cacheAlias = {};
    },
    walk: function walk(file) {
        if (this._cacheModules[file]) {
            return this._cacheModules[file];
        }

        var f = null,
            ext = _path2.default.extname(file),
            dir = void 0,
            modulePath = void 0,
            filename = void 0;
        for (var i = 0, l = this.modulePaths.length; i < l; i++) {
            modulePath = this.modulePaths[i];
            filename = _path2.default.join(modulePath, file);
            var tmp = filename;
            if (ext) {
                f = _util2.default.isFile(tmp);
            } else {
                tmp = filename + '.js';
                f = _util2.default.isFile(tmp);
            }
            if (!f && _util2.default.isDir(tmp)) {
                tmp = filename + _path2.default.sep + 'index.js';
                f = _util2.default.isFile(tmp);
            }
            if (f) {
                filename = tmp;
                break;
            }
        }
        if (!f) {
            return null;
        }
        this._cacheModules[file] = {
            modulePath: modulePath,
            file: filename
        };
        return this._cacheModules[file];
    },
    getPkg: function getPkg(lib) {
        var pkg = null,
            dir = null;
        var o = this.walk(lib + _path2.default.sep + 'package.json');
        if (!o) return null;

        pkg = _util2.default.readFile(o.file);
        if (pkg) {
            try {
                pkg = JSON.parse(pkg);
            } catch (e) {
                pkg = null;
            }
        }
        if (!pkg) return null;
        return {
            pkg: pkg,
            modulePath: o.modulePath,
            dir: _path2.default.join(o.modulePath, lib)
        };
    },
    getMainFile: function getMainFile(lib) {
        var o = this.getPkg(lib);
        if (!o) {
            return null;
        }
        var main = o.pkg.main || 'index.js';
        if (o.pkg.browser && typeof o.pkg.browser === 'string') {
            main = o.pkg.browser;
        }
        return {
            file: main,
            modulePath: o.modulePath,
            pkg: o.pkg,
            dir: o.dir
        };
    },
    resolveAlias: function resolveAlias(lib) {
        if (!this.alias) return lib;
        if (this._cacheAlias[lib]) {
            return this._cacheAlias[lib];
        }
        var rst = lib;

        for (var k in this.alias) {
            var alias = this.alias[k];
            if (k[k.length - 1] === '$') {
                k = k.substring(0, k.length - 1);
                if (k === lib) {
                    if (_path2.default.extname(alias) === '') {
                        // this is directory
                        this._cacheAlias[lib] = _path2.default.join(alias, 'index.js');
                    } else {
                        this._cacheAlias[lib] = alias;
                    }
                }
            } else {
                if (lib.indexOf(k) === 0 && lib === k || lib !== k && lib.indexOf(k + '/') === 0) {
                    this._cacheAlias[lib] = lib.replace(k, alias);
                }
            }
        }
        if (!this._cacheAlias[lib]) {
            this._cacheAlias[lib] = lib;
        }
        return this._cacheAlias[lib];
    }
};
//# sourceMappingURL=resolve.js.map