'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _hashSum = require('hash-sum');

var _hashSum2 = _interopRequireDefault(_hashSum);

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

var _resolve = require('./resolve');

var _resolve2 = _interopRequireDefault(_resolve);

var _escodegen = require('escodegen');

var _escodegen2 = _interopRequireDefault(_escodegen);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var acorn = require('acorn');
require('acorn-es7')(acorn);
require('acorn-es7-plugin')(acorn);
var acorn_walk = require('acorn/dist/walk');

_safe2.default.enabled = true;

_safe2.default.setTheme({
  /*SILLY: 'rainbow',
  INPUT: 'grey',
  VERBOSE: 'cyan',
  PROMPT: 'grey',
  INFO: 'green',
  DATA: 'grey',
  HELP: 'cyan',
  WARN: 'yellow',
  DEBUG: 'blue',
  ERROR: 'red',*/
  'info': 'grey',
  '变更': 'bgYellow',
  '删除': 'bgMagenta',
  '执行': 'blue',
  '压缩': 'blue',
  '信息': 'grey',
  '完成': 'green',
  '创建': 'green',
  '监听': 'magenta',
  '错误': 'red',
  '测试': 'red',
  '拷贝': 'yellow',
  '编译': 'blue',
  '写入': 'green'
});

var ID_CACHE = {};
var utils = {
  Log: function Log() {
    // console.log(colors.blue('--info-- '), ...msg)
  },
  seqPromise: function seqPromise(promises) {
    return new Promise(function (resolve, reject) {

      var count = 0;
      var results = [];

      var iterateeFunc = function iterateeFunc(previousPromise, currentPromise) {
        return previousPromise.then(function (result) {
          if (count++ !== 0) results = results.concat(result);
          return currentPromise(result, results, count);
        }).catch(function (err) {
          return reject(err);
        });
      };

      promises = promises.concat(function () {
        return Promise.resolve();
      });

      promises.reduce(iterateeFunc, Promise.resolve(false)).then(function (res) {
        resolve(results);
      });
    });
  },


  /*
  小程序内表达式和weex一致
  :prop="a"   -> prop="{{a}}"
  :prop="a&&b>0"  -> prop="{{a&&b>0}}" 
  :prop="a?b:c"  ->  prop="{{a?b:c}}"
  关于对象绑定
  :prop="{key:value}"   ->  prop="{{key:value}}"
   关于style绑定
  小程序不支持style的对象绑定
  :style="{fontSize:value}"  -> style="font-size:{{value}}"
   */
  parseStyleExp: function parseStyleExp(exp) {
    var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

    if (prefix && prefix[0] !== '$') {
      prefix = '$' + prefix + '$';
    }
    try {
      var doreplace = function doreplace(exp) {
        var ast = acorn.parse(exp);
        var exps = [];
        acorn_walk.simple(ast, {
          Expression: function Expression(node) {
            // utils.Log('Expression',escodegen.generate(node,{format: {compact:true}}));
          },
          Identifier: function Identifier(node) {
            // node.name =  '{{' + prefix + node.name + '}}'
            node.name = prefix + node.name;
          },
          Punctuator: function Punctuator(node) {
            if (node.value === ',') {
              node.value = ';';
            }
          },
          ObjectExpression: function ObjectExpression(node) {
            exps.push(node);
          },
          Property: function Property(node) {}
        });
        var node = exps[exps.length - 1];

        var s = '';
        node.properties.forEach(function (property) {
          var key = _escodegen2.default.generate(property.key, { format: { compact: true } });
          var value = _escodegen2.default.generate(property.value, { format: { compact: true } });
          key = key.replace(/([A-Z])/g, '-$1').toLowerCase();
          utils.Log('parseStyleExp key=', key);
          s = s + key + ': {{' + value + '}};';
        });
        return s;
        // return ast
      };

      if (exp.match(/^\s*{/)) {
        //like {width: variable, ...}
        //change object expression to assigenment expression
        //so that can be parsed
        exp = 'f=' + exp;
        // const ast = doreplace(exp)
        //strip assignement part
        // let r = escodegen.generate(ast.body[0].expression.right, {format: {compact:true}})

        var s = doreplace(exp);
        return s;
      } else {
        return this.parseExp(exp, prefix);
      }
    } catch (e) {
      utils.error('\u7F16\u8BD1\u6A21\u677F\u4E2D\u8868\u8FBE\u5F0F' + exp + '\u51FA\u9519\uFF1A' + e + e.stack);
    }
  },
  testExp: function testExp(exp) {
    try {
      var ast = acorn.parse(exp, {
        sourceType: 'module',
        ecmaVersion: 8,
        plugins: { asyncawait: true, es7: true }
      });

      var exps = [];

      acorn_walk.simple(ast, {
        Expression: function Expression(node) {
          utils.Log('Expression', _escodegen2.default.generate(node, { format: { compact: true } }));
        },
        Literal: function Literal(node) {
          utils.Log('Literal', _escodegen2.default.generate(node, { format: { compact: true } }));
          // utils.Log('Literal',node);
        },
        Identifier: function Identifier(node) {
          utils.Log('Identifier', _escodegen2.default.generate(node, { format: { compact: true } }));
          // utils.Log('Identifier',node);
          // node.name = 'this.' + node.name;
        },
        LabeledStatement: function LabeledStatement(node) {
          utils.Log('LabeledStatement', _escodegen2.default.generate(node, { format: { compact: true } }));
        },
        Functions: function Functions(node) {
          utils.Log('Functions', _escodegen2.default.generate(node, { format: { compact: true } }));
        },
        ObjectExpression: function ObjectExpression(node) {
          utils.Log('ObjectExpression', _escodegen2.default.generate(node, { format: { compact: true } }));
          // utils.Log('ObjectExpression',node);
          // utils.Log(node.properties[0]);
          // utils.Log(node.properties[1]);
          exps.push(node);
        },
        Property: function Property(node) {
          utils.Log('Property', _escodegen2.default.generate(node, { format: { compact: true } }));
          utils.Log('Property', node);
          // let v = node.value;
          // v.
        },
        MemberExpression: function MemberExpression(node) {
          // let v = node.property;
          utils.Log('MemberExpression', _escodegen2.default.generate(node, { format: { compact: true } }));
          // utils.Log('MemberExpression',node);
          //
          // utils.Log('MemberExpression',v);
          // v.name = v.name+'_weex_style';
        },
        Pattern: function Pattern(node) {
          utils.Log('Pattern', _escodegen2.default.generate(node, { format: { compact: true } }));
        }
      });
      var last = exps[exps.length - 1];
      // let arr = [];
      // last.properties.forEach(node=>{
      //     arr.push(node.key.name);
      // })
      // utils.Log('arr=',arr)

      return _escodegen2.default.generate(last, { format: { compact: true } });
    } catch (e) {
      utils.error('\u7F16\u8BD1\u6A21\u677F\u4E2D\u8868\u8FBE\u5F0F' + exp + '\u51FA\u9519\uFF1A' + e + e.stack);
    }
  },
  parseExp: function parseExp(exp) {
    var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

    try {
      var doreplace = function doreplace(exp) {
        var ast = acorn.parse(exp);
        acorn_walk.simple(ast, {
          Identifier: function Identifier(node) {
            node.name = prefix + node.name;
          }
        });
        return ast;
      };

      if (prefix && prefix[0] !== '$') {
        prefix = '$' + prefix + '$';
      }


      if (exp.match(/^\s*{/)) {
        //like {width: variable, ...}
        //change object expression to assigenment expression
        //so that can be parsed
        exp = 'f=' + exp;
        var ast = doreplace(exp);
        //strip assignement part
        var r = _escodegen2.default.generate(ast.body[0].expression.right, { format: { compact: true } });
        return '{' + r.replace(';', '') + '}';
      } else {
        //like variable | expression
        var _ast = doreplace(exp);
        var _r = _escodegen2.default.generate(_ast, { format: { compact: true } });
        return "{{" + _r.replace(';', '') + "}}";
      }
    } catch (e) {
      utils.error('\u7F16\u8BD1\u6A21\u677F\u4E2D\u8868\u8FBE\u5F0F' + exp + '\u51FA\u9519\uFF1A' + e + e.stack);
    }
  },
  exec: function exec(cmd, quite) {
    return new Promise(function (resolve, reject) {
      var fcmd = (0, _child_process.exec)(cmd, function (err, stdout, stderr) {
        if (err) {
          reject(err);
        } else {
          resolve(stdout, stderr);
        }
      });
      fcmd.stdout.on('data', function (chunk) {
        !quite && process.stdout.write(chunk);
      });
      fcmd.stderr.on('data', function (chunk) {
        !quite && process.stdout.write(chunk);
      });
    });
  },
  timeoutExec: function timeoutExec(sec, cmd, quite) {
    var timeout = new Promise(function (resolve, reject) {
      setTimeout(function () {
        reject('timeout');
      }, sec * 1000);
    });
    var task = this.exec(cmd, quite);
    return Promise.race([timeout, task]);
  },
  elemToArray: function elemToArray(elems) {
    var rst = [];
    for (var i = 0, len = elems.$$length; i < len; i++) {
      rst.push(elems[i]);
    }
    return rst;
  },
  elemRecursion: function elemRecursion(node) {
    var _this = this;

    // this.Log('elemRecursion',Object.keys(node), typeof node);
    var arr = [];
    [].slice.call(node || []).forEach(function (child) {
      // this.Log('foreach',child.nodeName)
      if (child.nodeName !== '#text' && child.nodeName !== '#comment') {
        if (child.parentNode) {}
        if (child.getAttribute('v-for')) {
          arr.push(child);
        }
        arr = arr.concat(_this.elemRecursion(child.childNodes));
      }
    });
    return arr;
  },
  getNodes: function getNodes(node) {
    var _this2 = this;

    var infor = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var keyword = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

    var arr = [];
    [].slice.call(node.childNodes || []).forEach(function (child) {
      if (child.nodeName !== '#text' && child.nodeName !== '#comment') {

        if (infor) {
          arr.push({
            node: child,
            infor: true,
            keyword: keyword
          });
        } else {
          arr.push({
            node: child,
            infor: false
          });
        }
        var f = false;
        if (child.getAttribute('for')) {
          f = true;
          var item = child.getAttribute('item');
          var index = child.getAttribute('index');
          var key = child.getAttribute('key');
          if (item && item !== '') {
            keyword.push(item);
          }
          if (index && index !== '') {
            keyword.push(index);
          }
          if (key && key !== '') {
            keyword.push(key);
          }
        }
        arr = arr.concat(_this2.getNodes(child, infor || f, keyword));
      }
    });
    return arr;
  },
  getComId: function getComId(elem) {
    var tagName = elem.nodeName;
    var path = elem.getAttribute('path');
    var id = elem.getAttribute('id');
    if (tagName !== 'component') return tagName;
    if (id) return id;
    if (path && !id) return path;
  },
  getComPath: function getComPath(elem) {
    return elem.getAttribute('path') || this.getComId(elem);
  },
  findComponentInTemplate: function findComponentInTemplate(com, template) {
    if (typeof com !== 'string') {
      com = this.getComId(com);
    }
    var definePath = template.components[com];
    definePath = definePath.indexOf('.') === -1 ? definePath : _path2.default.resolve(template.src, '..' + _path2.default.sep + definePath);
    return this.findComponent(definePath, true);
  },
  oCompRealPath: function oCompRealPath(com) {
    // console.log('oCompRealPath:', com)
    var src = '';
    if (this.isFile(com)) {
      var txt = this.readFile(com);
      var matches = /register\(require\s*\(\s*['"](.*)['"]\s*\)/.exec(txt);
      if (matches && matches[1]) {
        src = _path2.default.resolve(_path2.default.dirname(com), matches[1]);
      }
    } else if (this.isFile(com + '/index.js')) {
      var txt = this.readFile(com + '/index.js');
      var matches = /register\(require\s*\(\s*['"](.*)['"]\s*\)/.exec(txt);
      if (matches && matches[1]) {
        src = _path2.default.resolve(com, matches[1]);
      }
    } else if (this.isFile(com + '.js')) {
      var txt = this.readFile(com + '.js');
      var matches = /register\(require\s*\(\s*['"](.*)['"]\s*\)/.exec(txt);
      if (matches && matches[1]) {
        src = _path2.default.resolve(_path2.default.dirname(com), matches[1]);
      }
    }
    return src;
  },
  findComponent: function findComponent(com) {
    var wpyExt = _cache2.default.getExt();

    var src = '';
    if (com.indexOf(_path2.default.sep) !== -1) {
      if (com.match(wpyExt + '$')) {
        src = com;
      } else if (this.isFile(com + wpyExt)) {
        src = com + wpyExt;
      } else if (this.isFile(com + '/index.js')) {
        var txt = this.readFile(com + '/index.js');
        var matches = /register\(require\s*\(\s*['"](.*)['"]\s*\)/.exec(txt);
        if (matches && matches[1]) {
          src = _path2.default.resolve(com, matches[1]);
        }
      } else if (this.isFile(com + '.js')) {
        var txt = this.readFile(com + '.js');
        var matches = /register\(require\s*\(\s*['"](.*)['"]\s*\)/.exec(txt);
        if (matches && matches[1]) {
          src = _path2.default.resolve(_path2.default.dirname(com), matches[1]);
        }
      }
    } else {
      console.log('com else', com);
      var o = _resolve2.default.getMainFile(com);
      console.log(o);
      if (o) {
        src = _path2.default.join(o.dir, o.file);
      } else {
        var comPath = _path2.default.join(this.currentDir, _cache2.default.getSrc(), 'components', com);
        if (this.isFile(comPath + wpyExt)) {
          src = comPath + wpyExt;
        } else if (this.isFile(comPath + '/index' + wpyExt)) {
          src = comPath + '/index' + wpyExt;
        } else if (this.isFile(comPath + '/' + com + wpyExt)) {
          src = comPath + '/' + com + wpyExt;
        }
      }
    }
    return src;
  },


  currentDir: process.cwd(),
  cliDir: __dirname,

  isFunction: function isFunction(fn) {
    return typeof fn === 'function';
  },
  isString: function isString(obj) {
    return toString.call(obj) === '[object String]';
  },
  isObject: function isObject(obj) {
    return toString.call(obj) === '[object Object]';
  },
  isNumber: function isNumber(obj) {
    return toString.call(obj) === '[object Number]';
  },
  isBoolean: function isBoolean(obj) {
    return toString.call(obj) === '[object Boolean]';
  },
  isArray: function isArray(obj) {
    return Array.isArray(obj);
  },
  isFile: function isFile(p) {
    p = (typeof p === 'undefined' ? 'undefined' : _typeof(p)) === 'object' ? _path2.default.join(p.dir, p.base) : p;
    if (!_fs2.default.existsSync(p)) {
      return false;
    }
    return _fs2.default.statSync(p).isFile();
  },
  isDir: function isDir(p) {
    if (!_fs2.default.existsSync(p)) {
      return false;
    }
    return _fs2.default.statSync(p).isDirectory();
  },

  /**
  * Hyphenate a camelCase string.
  */
  hyphenate: function hyphenate(str) {
    return str.replace(/([^-])([A-Z])/g, '$1-$2').replace(/([^-])([A-Z])/g, '$1-$2').toLowerCase();
  },

  /**
   * Camelize a hyphen-delimited string.
   */
  camelize: function camelize(str) {
    return str.replace(/-(\w)/g, function (_, c) {
      return c ? c.toUpperCase() : '';
    });
  },

  /**
   * xml dom 对 TEXT_NODE 和 ATTRIBUTE_NODE 进行转义。
   */
  decode: function decode(content) {
    var pmap = ['<', '&', '"'];
    var amap = ['&lt;', '&amp;', '&quot;'];
    var reg = new RegExp('(' + amap[0] + '|' + amap[1] + '|' + amap[2] + ')', 'ig');
    return content.replace(reg, function (match, m) {
      return pmap[amap.indexOf(m)];
    });
  },
  encode: function encode(content, start, end) {
    start = start || 0;
    end = end || content.length;

    var buffer = [];
    var pmap = ['<', '&', '"'];
    var amap = ['&lt;', '&amp;', '&quot;'];

    var i = 0,
        c = void 0;
    for (var _i = 0, len = content.length; _i < len; _i++) {
      if (_i < start || _i > end) {
        buffer.push(content[_i]);
      } else {
        c = pmap.indexOf(content[_i]);
        buffer.push(c === -1 ? content[_i] : amap[c]);
      }
    }
    return buffer.join('');
  },

  /**
   * xml replace
   */
  attrReplace: function attrReplace(content) {
    var config = utils.getConfig();
    var knownTags = [
    // 视图容器
    'view', 'scroll-view', 'swiper', 'swiper-item',

    // 基础内容
    'icon', 'text', 'progress',

    // 表单组件
    'button', 'checkbox-group', 'checkbox', 'form', 'input', 'label', 'picker', 'picker-view', 'picker-view-column', 'radio-group', 'radio', 'slider', 'switch', 'textarea',

    // 导航
    'navigator',

    // 媒体组件
    'audio', 'image', 'video',

    // 地图
    'map',

    // 画布
    'canvas',

    // 客服会话
    'contact-button'];
    var configTags = config.knownTags;
    if (configTags && Array.isArray(configTags)) {
      // 增加可配置的 knownTags
      // 以防止由于小程序升级 上边的默认tag不完全导致的问题
      // 如果说小程序升级的话 在 wepy 没升级的情况下
      // 用户可以通过临时加入 knownTags 的方法兼容
      knownTags.push.apply(knownTags, configTags);
    }
    // Exp error when using this code: <input focus wx:if="{{test >
    if (config.output === 'ant') {
      content = content.replace(/\s+wx\:(\w+)/ig, function (match, name) {
        return ' a:' + name;
      });
    }
    return content.replace(/<([\w-]+)\s*[\s\S]*?(\/|<\/[\w-]+)>/ig, function (tag, tagName) {
      tagName = tagName.toLowerCase();
      return tag.replace(/\s+:([\w-_]*)([\.\w]*)\s*=/ig, function (attr, name, type) {
        // replace :param.sync => v-bind:param.sync
        if (type === '.once' || type === '.sync') {} else type = '.once';
        return ' v-bind:' + name + type + '=';
      }).replace(/\s+\@([\w-_]*)([\.\w]*)\s*=/ig, function (attr, name, type) {
        // replace @change => v-on:change

        var eventMap = {
          click: 'tap',
          focus: 'focus',
          blur: 'blur',
          return: 'confirm',
          input: 'input',
          change: 'change',
          scroll: 'scroll',
          loadmore: 'loadmore',
          touchstart: 'touchstart',
          touchmove: 'touchmove',
          touchend: 'touchend',
          touchcancel: 'touchcancel'
        };
        var prefix = void 0;

        if (!eventMap[name]) {
          prefix = 'v-on:';
        } else {
          if (type === '.stop') {
            prefix = 'catch';
          } else {
            prefix = 'bind';
          }
          name = eventMap[name];
        }

        // let prefix = type !== '.user' ? (type === '.stop' ? 'catch' : 'bind') : 'v-on:';
        // if (config.output === 'ant' && prefix === 'bind') {
        //     prefix = 'on';
        //     name = name[0].toUpperCase() + name.substring(1);
        // }
        return ' ' + prefix + name + '=';
      });
    });
  },

  /**
   * get indent of a mutiple lines string
   * return {length: 4, char: ' '}
   */
  getIndent: function getIndent(str) {
    var arr = str.split('\n');
    while (arr.length && !/\w/.test(arr[0])) {
      // if the first line is empty line, then get rid of it
      arr.shift();
    }
    var indent = { firstLineIndent: 0, indent: 0, char: '' };
    var s = arr[0],
        i = 0;
    if (s.charCodeAt(0) === 32 || s.charCodeAt(0) === 9) {
      // 32 is space, 9 is tab
      indent.char = s[0];
    }
    while (s[i] === indent.char) {
      i++;
    }
    indent.firstLineIndent = i;
    if (!arr[1]) return indent;
    s = arr[1], i = 0;
    if (!indent.char) {
      if (s.charCodeAt(0) === 32 || s.charCodeAt(0) === 9) {
        // 32 is space, 9 is tab
        indent.char = s[0];
      }
    }
    while (s[i] === indent.char) {
      i++;
    }
    indent.indent = i - indent.firstLineIndent;
    return indent;
  },

  /**
   * Fix indent for a mutiple lines string
   * @param  {String} str  string to fix
   * @param  {Number} num  4 means add 4 chars to each line, -4 means remove 4 chars for each line
   * @param  {String} char space or tab, indent charactor
   * @return {String}      fixed indent string
   */
  fixIndent: function fixIndent(str, num, char) {
    if (char === undefined) {
      var indent = getIndent(str);
      char = indent.char;
    }
    var arr = str.split('\n');
    if (num > 0) {
      // added char to each line
      arr.forEach(function (v, i) {
        var p = 0;
        while (p++ < num) {
          arr[i] = char + arr[i];
        }
      });
    } else {
      // remove char for each line
      arr.forEach(function (v, i) {
        arr[i] = arr[i].substr(-1 * num);
      });
    }
    return arr.join('\n');
  },
  unique: function unique(arr) {
    var tmp = {},
        out = [];
    arr.forEach(function (v) {
      if (!tmp[v]) {
        tmp[v] = 1;
        out.push(v);
      }
    });
    return out;
  },
  unlink: function unlink(p) {
    var rst = '';
    p = (typeof p === 'undefined' ? 'undefined' : _typeof(p)) === 'object' ? _path2.default.join(p.dir, p.base) : p;
    try {
      rst = _fs2.default.unlinkSync(p);
    } catch (e) {
      rst = null;
    }
    return rst;
  },
  readFile: function readFile(p) {
    var rst = '';
    p = (typeof p === 'undefined' ? 'undefined' : _typeof(p)) === 'object' ? _path2.default.join(p.dir, p.base) : p;
    try {
      rst = _fs2.default.readFileSync(p, 'utf-8');
    } catch (e) {
      rst = null;
    }
    return rst;
  },
  mkdir: function mkdir(name) {
    var rst = true;
    try {
      _fs2.default.mkdirSync(name);
    } catch (e) {
      rst = e;
    }
    return rst;
  },
  writeFile: function writeFile(p, data) {
    var opath = this.isString(p) ? _path2.default.parse(p) : p;
    if (!this.isDir(opath.dir)) {
      _mkdirp2.default.sync(opath.dir);
    }
    _fs2.default.writeFileSync(p, data);
  },
  copy: function copy(opath, ext, src, dist) {
    var target = this.getDistPath(opath, ext, src, dist);
    this.writeFile(target, this.readFile(_path2.default.join(opath.dir, opath.base)));
    var readable = _fs2.default.createReadStream(_path2.default.join(opath.dir, opath.base));
    var writable = _fs2.default.createWriteStream(target);
    readable.pipe(writable);
  },
  getRelative: function getRelative(opath) {
    return _path2.default.relative(this.currentDir, _path2.default.join(opath.dir, opath.base));
  },
  getDistPath: function getDistPath(opath, ext, src, dist) {
    var relative = void 0;
    src = src || _cache2.default.getSrc();
    dist = dist || _cache2.default.getDist();
    if (typeof opath === 'string') opath = _path2.default.parse(opath);
    ext = ext ? ext[0] === '.' ? ext : '.' + ext : opath.ext;
    // 第三组件
    if (opath.npm) {
      relative = _path2.default.relative(opath.npm.modulePath, opath.npm.dir);
      relative = _path2.default.join('npm', relative);
    } else {
      relative = _path2.default.relative(_path2.default.join(this.currentDir, src), opath.dir);
    }
    return _path2.default.join(this.currentDir, dist, relative, opath.name + ext);
  },
  getModifiedTime: function getModifiedTime(p) {
    return this.isFile(p) ? +_fs2.default.statSync(p).mtime : false;
  },
  getConfig: function getConfig() {
    var config = _cache2.default.getConfig();
    if (config) return config;

    var configFile = _path2.default.join(this.currentDir, _path2.default.sep, 'wekit.config.js');
    var configType = 'js';

    if (!this.isFile(configFile)) {
      configFile = _path2.default.join(this.currentDir, _path2.default.sep, '.wepyrc');

      config = this.readFile(configFile);
      try {
        config = JSON.parse(config);
      } catch (e) {
        config = null;
      }
    } else {
      config = require(configFile);
    }

    _cache2.default.setConfig(config);
    return config;
  },
  getIgnore: function getIgnore() {
    var ignoreFile = _path2.default.join(this.currentDir, _path2.default.sep, '.wepyignore');
    return this.isFile(ignoreFile) ? this.readFile(ignoreFile) : '';
  },
  getFiles: function getFiles() {
    var _this3 = this;

    var dir = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.cwd();
    var prefix = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

    var cfiles = _cache2.default.getFileList(dir);
    if (cfiles) return cfiles;
    dir = _path2.default.normalize(dir);
    if (!_fs2.default.existsSync(dir)) {
      return [];
    }
    var files = _fs2.default.readdirSync(dir);
    var rst = [];
    files.forEach(function (item) {
      var filepath = dir + _path2.default.sep + item;
      var stat = _fs2.default.statSync(filepath);
      if (stat.isFile()) {
        rst.push(prefix + item);
      } else if (stat.isDirectory()) {
        rst = rst.concat(_this3.getFiles(_path2.default.normalize(dir + _path2.default.sep + item), _path2.default.normalize(prefix + item + _path2.default.sep)));
      }
    });

    _cache2.default.setFileList(dir, rst);
    return rst;
  },
  getVersion: function getVersion() {
    var filepath = _path2.default.resolve(__dirname, '../package.json');
    var version = JSON.parse(this.readFile(filepath)).version;
    return version;
  },
  datetime: function datetime() {
    var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
    var format = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'HH:mm:ss';

    var fn = function fn(d) {
      return ('0' + d).slice(-2);
    };
    if (date && this.isString(date)) {
      date = new Date(Date.parse(date));
    }
    var formats = {
      YYYY: date.getFullYear(),
      MM: fn(date.getMonth() + 1),
      DD: fn(date.getDate()),
      HH: fn(date.getHours()),
      mm: fn(date.getMinutes()),
      ss: fn(date.getSeconds())
    };
    return format.replace(/([a-z])\1+/ig, function (a) {
      return formats[a] || a;
    });
  },
  error: function error(msg) {
    this.log(msg, 'error', false);
    process.exit(0);
  },
  warning: function warning(msg) {
    this.log(msg, 'warning', false);
  },
  log: function log(msg, type) {
    var showTime = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

    var dateTime = showTime ? _safe2.default.gray('[' + this.datetime() + '] ') : '';
    if (this.isObject(msg) || this.isArray(msg)) {
      msg = JSON.stringify(msg);
    }
    if (type && this.isString(type)) {
      type = type.toUpperCase();
      if (type === 'ERROR') {
        console.error(_safe2.default.red('[Error] ' + msg));
      } else if (type === 'WARNING') {
        console.error(_safe2.default.yellow('[WARNING] ' + msg));
      } else {
        var fn = _safe2.default[type] ? _safe2.default[type] : _safe2.default['info'];
        console.log(dateTime + fn('[' + type + ']') + ' ' + msg);
      }
    } else {
      console.log(dateTime + msg);
    }
  },
  output: function output(type, file, flag) {
    if (!flag) {
      flag = file.substr(file.lastIndexOf('.') + 1).toUpperCase();
      if (flag.length < 4) {
        var i = 4 - flag.length;
        while (i--) {
          flag += ' ';
        }
      }
    }
    this.log(flag + ': ' + _path2.default.relative(this.currentDir, file), type);
  },
  genId: function genId(filepath) {
    if (!ID_CACHE[filepath]) {
      ID_CACHE[filepath] = '_' + (0, _hashSum2.default)(filepath).slice(1, 8);
    }
    return ID_CACHE[filepath];
  }
};
exports.default = utils;
//# sourceMappingURL=util.js.map