import path from 'path';
import util from './util';
import cache from './cache';
import cWpy from './compile-wpy';

import loader from './loader';

import resolve from './resolve';


const currentPath = util.currentDir;

let appPath, npmPath, src, dist;

export default {
    resolveDeps (code, type, opath) {

        util.Log('resolveDeps,opath=',opath.base);
        let params = cache.getParams();
        let config = cache.getConfig();
        let wpyExt = params.wpyExt;
        let npmInfo = opath.npm;

        

        return code.replace(/require\(['"]([\w\d_\-\.\/@]+)['"]\)/ig, (match, lib) => {

            let resolved = lib;

            util.Log('require lib',lib);

            let target = '', source = '', ext = '', needCopy = false;

            if (config.output === 'ant' && lib === 'wepy') {
                lib = 'wepy-ant';
            }
            lib = resolve.resolveAlias(lib);
            if (path.isAbsolute(lib)) {
                source = lib;
                target = util.getDistPath(source);
            } else if (lib[0] === '.') { // require('./something'');
                source = path.join(opath.dir, lib);  // e:/src/util
                if (type === 'npm') {
                    target = path.join(npmPath, path.relative(npmInfo.modulePath, source));
                    needCopy = true;
                } else {
                    // e:/dist/util
                    target = util.getDistPath(source);
                    needCopy = false;
                }
            } else if (lib.indexOf('/') === -1 || // require('asset');
                lib.indexOf('/') === lib.length - 1 || // reqiore('a/b/something/')
                (lib[0] === '@' && lib.indexOf('/') !== -1 && lib.lastIndexOf('/') === lib.indexOf('/')) // require('@abc/something')
            ) {  

                let mainFile = resolve.getMainFile(lib);

                if (!mainFile) {
                    throw Error('找不到模块: ' + lib + '\n被依赖于: ' + path.join(opath.dir, opath.base) + '。\n请尝试手动执行 npm install ' + lib + ' 进行安装。');
                }
                npmInfo = {
                    lib: lib,
                    dir: mainFile.dir,
                    modulePath: mainFile.modulePath,
                    file: mainFile.file
                };
                source = path.join(mainFile.dir, mainFile.file);
                target = path.join(npmPath, lib, mainFile.file);

                lib += path.sep + mainFile.file;
                ext = '';
                needCopy = true;
            } else { // require('babel-runtime/regenerator')
                let o = resolve.walk(lib);
                source = path.join(o.modulePath, lib);
                target = path.join(npmPath, lib);
                ext = '';
                needCopy = true;
            }

            if (util.isFile(source + wpyExt)) {
                ext = '.js';
            } else if (util.isFile(source + '.js')) {
                ext = '.js';
            } else if (util.isDir(source) && util.isFile(source + path.sep + 'index.js')) {
                ext = path.sep + 'index.js';
            }else if (util.isFile(source)) {
                ext = '';
            } else {
                throw ('找不到文件: ' + source);
            }
            source += ext;
            target += ext;
            lib += ext;
            resolved = lib;    
            

            // 第三方组件
            if (/\.wpy$/.test(resolved)) {
                target = target.replace(/\.wpy$/, '') + '.js';
                resolved = resolved.replace(/\.wpy$/, '') + '.js';
                lib = resolved;
            }

            if (needCopy) {
                if (!cache.checkBuildCache(source)) {
                    cache.setBuildCache(source);
                    util.log('依赖: ' + path.relative(process.cwd(), target), '拷贝');
                    let newOpath = path.parse(source);
                    newOpath.npm = npmInfo;
                    this.compile('js', null, 'npm', newOpath);

                }
            }
            if (type === 'npm') {
                if (lib[0] !== '.') {
                    resolved = path.join('..' + path.sep, path.relative(opath.dir, npmInfo.modulePath), lib);
                } else {
                    if (lib[0] === '.' && lib[1] === '.')
                        resolved = './' + resolved;
                }

            } else {
                resolved = path.relative(util.getDistPath(opath, opath.ext, src, dist), target);
            }
            resolved = resolved.replace(/\\/g, '/').replace(/^\.\.\//, './');

            //处理ocomp组件规则
            const ocomp_path = util.oCompRealPath(source);
            if (ocomp_path) {
                // console.log("ocomp_path:", ocomp_path)
                resolved = ocomp_path.replace(/\.vue/i, ".js")
                resolved = path.relative(opath.dir, resolved)
            } 

            return `require('${resolved}')`;
        });
    },

    npmHack (opath, code) {
        // 一些库（redux等） 可能会依赖 process.env.NODE_ENV 进行逻辑判断
        // 这里在编译这一步直接做替换 否则报错
        code = code.replace(/process\.env\.NODE_ENV/g, JSON.stringify(process.env.NODE_ENV));
        switch(opath.base) {
            case 'lodash.js':
            case '_global.js':
                code = code.replace('Function(\'return this\')()', 'this');
                break;
            case '_html.js':
                code = 'module.exports = false;';
                break;
            case '_microtask.js':
                code = code.replace('if(Observer)', 'if(false && Observer)');
                // IOS 1.10.2 Promise BUG
                code = code.replace('Promise && Promise.resolve', 'false && Promise && Promise.resolve');
                break;
            case '_freeGlobal.js':
                code = code.replace('module.exports = freeGlobal;', 'module.exports = freeGlobal || this;')
        }
        let config = util.getConfig();
        if (config.output === 'ant' && opath.dir.substr(-19) === 'wepy-async-function') {
            code = '';
        }
        return code;
    },

    compile (lang, code, type, opath) {
        let config = util.getConfig();

        src = cache.getSrc();
        dist = cache.getDist();
        npmPath = path.join(currentPath, dist, 'npm' + path.sep);

        if (!code) {
            code = util.readFile(path.join(opath.dir, opath.base));
            if (code === null) {
                throw '打开文件失败: ' + path.join(opath.dir, opath.base);
            }
        }

        let compiler = loader.loadCompiler(lang);

        if (!compiler) {
            return;
        }
        
        // console.log('编译js:', opath.base);
        //todo: 不要编译组件index.js
        
        code = code.replace(/暴露[\n\s\r]*?import { register } from 'utils'/g, '')
	
		if (opath.base.match(/\.json$/)) {
			util.Log('json', opath);
			code = 'module.exports = ' + code
		}
        
        let js = code
        compiler(code, config.compilers[lang] || {}).then(compileResult => {
            // console.log("compileResult for ", opath)
            let sourceMap;
            if (typeof(compileResult) === 'string') {
                code = compileResult;
            } else {
                sourceMap = compileResult.map;
                code = compileResult.code;
            }
            
            util.Log('opath ,type',opath.base,type,type==='page');
            if (type !== 'npm') {
                if (type === 'page' || type === 'app') {
                    code = code.replace(/(export\s+default\s+|module\.exports\s+=\s+|exports\.default\s+=\s+){((\n|.)*)}/g, function (m, exports, code) {
                        let s = `var $export$${opath.name} = {${code}}`

						// util.Log('exports', opath.name, exports, code)
						// console.log('xxopath1',opath.base);
						if (type === 'page') {
							let pagePath = path.join(path.relative(appPath.dir, opath.dir), opath.name).replace(/\\/ig, '/');
                            s = s + `\nPage(require('wekit').default.$createPage($export$${opath.name} , '${pagePath}'));\n`;
						} else {
							appPath = opath;
							let appConfig = JSON.stringify(config.appConfig);
                            s = s + `\nApp(require('wekit').default.$createApp( $export$${opath.name}, ${appConfig}));\n`;
						}s

                        return s
					})

                    code = code.replace(/exports\.default\s*=\s*(\w+);/ig, function (m, defaultExport) {
						// util.Log('opath',code,type, opath.dir, opath.base)
                        // util.Log(m, defaultExport)
                        if (defaultExport === 'undefined') {
                            return '';
                        }
                        if (type === 'page') {
                            let pagePath = path.join(path.relative(appPath.dir, opath.dir), opath.name).replace(/\\/ig, '/');
                            return `\nPage(require('wekit').default.$createPage(${defaultExport} , '${pagePath}'));\n`;
                        } else {
                            appPath = opath;
                            let appConfig = JSON.stringify(config.appConfig);
                            return `\nApp(require('wekit').default.$createApp(${defaultExport}, ${appConfig}));\n`;
                        }
                    });
                }
            }

            // util.Log('resolveDeps, code=',code);
            code = this.resolveDeps(code, type, opath);

            if (type === 'npm' && opath.ext === '.wpy') { // 第三方npm组件，后缀恒为wpy
                cWpy.compile(opath);
                return;
            }
            let target;
            if (type !== 'npm') {
                target = util.getDistPath(opath, 'js');
            } else {
                code = this.npmHack(opath, code);
                target = path.join(npmPath, path.relative(opath.npm.modulePath, path.join(opath.dir, opath.base)));
            }
            if (sourceMap) {
                sourceMap.sources = [opath.name + '.js'];
                sourceMap.file = opath.name + '.js';
                var Base64 = require('js-base64').Base64;
                code += `\r\n//# sourceMappingURL=data:application/json;charset=utf-8;base64,${Base64.encode(JSON.stringify(sourceMap))}`;
            }
	
	
			if (opath.base.match(/\.json$/)) {
				target = path.join(opath.dir, opath.base + '.js');
				util.Log('target', target);
			}
           
            
            let plg = new loader.PluginHelper(config.plugins, {
                type: type,
                code: code,
                file: target,
                output (p) {
                    util.output(p.action, p.file);
                },
                done (result) {
                    util.output('写入', result.file);
                    util.writeFile(target, result.code);
                }
            });
            // 缓存文件修改时间戳
            cache.saveBuildCache();
        }).catch((e) => {
            util.error('compile-script.js:'+ e + e.stack + '\n 源码:' + code);
            process.exit(0);
        });
    }

}
